diff --git a/lib/extras.py b/lib/extras.py
index 3fd1fb0..60e4c7f 100644
--- a/lib/extras.py
+++ b/lib/extras.py
@@ -816,7 +816,9 @@ class CompositeCaster(object):
 
     .. attribute:: type
 
-        The type of the Python objects returned. If :py:func:`collections.namedtuple()`
+        The type of the Python objects returned. If a ctor argument is provided,
+        then this type will be used at instantiation. If the ctor argument is
+        None (default) and :py:func:`collections.namedtuple()`
         is available, it is a named tuple with attributes equal to the type
         components. Otherwise it is just the `!tuple` object.
 
@@ -829,14 +831,19 @@ class CompositeCaster(object):
         List of component type oids of the type to be casted.
 
     """
-    def __init__(self, name, oid, attrs, array_oid=None):
+    def __init__(self, name, oid, attrs, array_oid=None, ctor=None):
         self.name = name
         self.oid = oid
         self.array_oid = array_oid
 
         self.attnames = [ a[0] for a in attrs ]
         self.atttypes = [ a[1] for a in attrs ]
-        self._create_type(name, self.attnames)
+        # Defaults to named tuple
+        if ctor is None:
+            self._create_type(name, self.attnames)
+        else:
+            self._ctor = ctor
+            self.type = ctor
         self.typecaster = _ext.new_type((oid,), name, self.parse)
         if array_oid:
             self.array_typecaster = _ext.new_array_type(
@@ -853,9 +860,10 @@ class CompositeCaster(object):
             raise psycopg2.DataError(
                 "expecting %d components for the type %s, %d found instead" %
                 (len(self.atttypes), self.name, len(tokens)))
-
-        return self._ctor(curs.cast(oid, token)
-            for oid, token in zip(self.atttypes, tokens))
+        kwargs = dict((key, curs.cast(oid, token))
+                      for key, oid, token in
+                      zip(self.attnames, self.atttypes, tokens))
+        return self._ctor(**kwargs)
 
     _re_tokenize = regex.compile(r"""
   \(? ([,)])                        # an empty token, representing NULL
@@ -888,10 +896,13 @@ class CompositeCaster(object):
             self._ctor = self.type
         else:
             self.type = namedtuple(name, attnames)
-            self._ctor = self.type._make
+
+            def tuple_ctor(**kwargs):
+                return self.type(*[kwargs.get(key) for key in self.attnames])
+            self._ctor = tuple_ctor
 
     @classmethod
-    def _from_db(self, name, conn_or_curs):
+    def _from_db(self, name, conn_or_curs, ctor=None):
         """Return a `CompositeCaster` instance for the type *name*.
 
         Raise `ProgrammingError` if the type is not found.
@@ -938,10 +949,12 @@ ORDER BY attnum;
         type_attrs = [ (r[2], r[3]) for r in recs ]
 
         return CompositeCaster(tname, type_oid, type_attrs,
-            array_oid=array_oid)
+                               array_oid=array_oid, ctor=ctor)
 
-def register_composite(name, conn_or_curs, globally=False):
-    """Register a typecaster to convert a composite type into a tuple.
+def register_composite(name, conn_or_curs, globally=False, ctor=None,
+                       register_adapter=False):
+    """Register a typecaster to convert a composite type into a tuple, or a
+    user-provided type.
 
     :param name: the name of a PostgreSQL composite type, e.g. created using
         the |CREATE TYPE|_ command
@@ -950,6 +963,11 @@ def register_composite(name, conn_or_curs, globally=False):
         object, unless *globally* is set to `!True`
     :param globally: if `!False` (default) register the typecaster only on
         *conn_or_curs*, otherwise register it globally
+    :param ctor: if not None, instantiate this type when converting the
+        composite type, and register an adapter as well for this type. If None
+        (default), return a named-tuple for the composite type.
+    :param register_adapter: if true, and ctor is not None, register an adapter
+        for the ctor type. (default false).
     :return: the registered `CompositeCaster` instance responsible for the
         conversion
 
@@ -957,11 +975,21 @@ def register_composite(name, conn_or_curs, globally=False):
         added support for array of composite types
 
     """
-    caster = CompositeCaster._from_db(name, conn_or_curs)
+    caster = CompositeCaster._from_db(name, conn_or_curs, ctor=ctor)
     _ext.register_type(caster.typecaster, not globally and conn_or_curs or None)
+    if ctor is not None and register_adapter:
+        # Register an adapter as well
+        class CtorAdapter(_ext.SQL_IN):
+
+            def __init__(self, instance):
+                super(CtorAdapter, self).__init__((getattr(instance, key) for
+                                                   key in caster.attnames))
+        CtorAdapter.__name__ = ctor.__name__ + 'Adapter'
+        _ext.register_adapter(ctor, CtorAdapter)
 
     if caster.array_typecaster is not None:
-        _ext.register_type(caster.array_typecaster, not globally and conn_or_curs or None)
+        _ext.register_type(caster.array_typecaster, not globally and
+                           conn_or_curs or None)
 
     return caster
 
diff --git a/tests/test_types_extras.py b/tests/test_types_extras.py
index 49be390..9e75971 100755
--- a/tests/test_types_extras.py
+++ b/tests/test_types_extras.py
@@ -557,6 +557,49 @@ class AdaptTypeTestCase(unittest.TestCase):
             self.assertEqual(v.adate, date(2011,1,2))
 
     @skip_if_no_composite
+    def test_cast_composite_ctor(self):
+        self._create_type("type_is",
+                          [("anint", "integer"), ("astring", "text")])
+
+        class TypeIs(object):
+
+            def __init__(self, anint=None, astring=None):
+                self.anint = anint
+                self.astring = astring
+
+        psycopg2.extras.register_composite("type_is", self.conn, ctor=TypeIs,
+                                           register_adapter=True)
+        curs = self.conn.cursor()
+        r = (2, "hello")
+        curs.execute("select %s::type_is;", (r,))
+        v = curs.fetchone()[0]
+        assert isinstance(v, TypeIs)
+        assert v.anint == 2
+        assert v.astring == "hello"
+        r = TypeIs(anint=2, astring="hello")
+        curs.execute("select %s::type_is;", (r,))
+        v = curs.fetchone()[0]
+        assert isinstance(v, TypeIs)
+        assert v.anint == 2
+        assert v.astring == "hello"
+
+    @skip_if_no_composite
+    def test_cast_composite_dict(self):
+        self._create_type("type_is",
+                          [("anint", "integer"), ("astring", "text")])
+        psycopg2.extras.register_composite("type_is", self.conn, ctor=dict)
+        r = (2, 'hello')
+        curs = self.conn.cursor()
+        curs.execute("select %s::type_is;", (r,))
+        v = curs.fetchone()[0]
+        assert isinstance(v, dict)
+        assert v['anint'] == 2
+        assert v['astring'] == "hello"
+
+
+
+
+    @skip_if_no_composite
     def test_cast_nested(self):
         self._create_type("type_is",
             [("anint", "integer"), ("astring", "text")])
